/*-------------------------------------------------------------------------+
| Copyright 2012 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editor;

import static java.lang.Character.isLetterOrDigit;
import static org.eclipse.jface.text.IDocument.DEFAULT_CONTENT_TYPE;
import static org.eclipse.wb.swt.SWTResourceManager.getColor;

import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.text.presentation.IPresentationReconciler;
import org.eclipse.jface.text.presentation.PresentationReconciler;
import org.eclipse.jface.text.rules.DefaultDamagerRepairer;
import org.eclipse.jface.text.rules.IRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.ITokenScanner;
import org.eclipse.jface.text.rules.IWordDetector;
import org.eclipse.jface.text.rules.RuleBasedScanner;
import org.eclipse.jface.text.rules.WordRule;
import org.eclipse.jface.text.source.IAnnotationHover;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.SourceViewerConfiguration;
import org.eclipse.swt.graphics.Color;
import org.fortiss.tooling.base.ui.editor.annotations.AnnotationHover;

/**
 * Class responsible for proper configuration of the {@link SourceEditorBase}.
 * 
 * @author doebber
 */
public abstract class SourceEditorConfigurationBase<T extends EObject>
		extends SourceViewerConfiguration {

	/** Stores the actual editor object. */
	protected SourceEditorBase<T> editor;

	/** Constructor. */
	public SourceEditorConfigurationBase(SourceEditorBase<T> editor) {
		this.editor = editor;
	}

	/** Color constant used to display code in dark red. */
	protected Color DARK_RED = getColor(128, 0, 0);

	/** Color constant used to display code in dark blue. */
	protected Color DARK_BLUE = getColor(0, 0, 128);

	/** Color constant used to display code in green. */
	protected Color GREEN = getColor(0, 178, 24);

	/** Identifier for the start of a comment. */
	public static final String COMMENT_START_IDENTIFIER = "/*";

	/** Identifier for the end of a comment. */
	public static final String COMMENT_END_IDENTIFIER = "*/";

	/** Detector used to create the scanners. */
	protected IWordDetector detector = new IWordDetector() {
		/** {@inheritDoc} */
		@Override
		public boolean isWordStart(char c) {
			return isLetterOrDigit(c) || COMMENT_START_IDENTIFIER.indexOf(c) != -1 ||
					COMMENT_END_IDENTIFIER.indexOf(c) != -1;
		}

		/** {@inheritDoc} */
		@Override
		public boolean isWordPart(char c) {
			return isLetterOrDigit(c) || c == '_' || COMMENT_START_IDENTIFIER.indexOf(c) != -1 ||
					COMMENT_END_IDENTIFIER.indexOf(c) != -1;
		}
	};

	/** {@inheritDoc} */
	@Override
	public IPresentationReconciler getPresentationReconciler(ISourceViewer sourceViewer) {
		PresentationReconciler reconciler = new PresentationReconciler();

		DefaultDamagerRepairer dr = new DefaultDamagerRepairer(getScanner());
		reconciler.setDamager(dr, DEFAULT_CONTENT_TYPE);
		reconciler.setRepairer(dr, DEFAULT_CONTENT_TYPE);

		return reconciler;
	}

	/** Returns the scanner. */
	private ITokenScanner getScanner() {
		return getScannerForSyntaxHighlighting();
	}

	/** Returns the annotation hover. */
	@Override
	public IAnnotationHover getAnnotationHover(ISourceViewer sourceViewer) {
		return new AnnotationHover(editor.getAnnotationModel());
	}

	/** Returns the rule based scanner. */
	private RuleBasedScanner getScannerForSyntaxHighlighting() {
		List<WordRule> rules = getCommonRules();
		rules.add(getRuleSpecificToEditor());
		RuleBasedScanner scanner = new RuleBasedScanner();
		scanner.setRules(rules.toArray(new IRule[0]));
		return scanner;
	}

	/** Returns a list of rules common for the syntax highlighting. */
	abstract protected List<WordRule> getCommonRules();

	/** Returns a rule specific to the editor. */
	abstract protected WordRule getRuleSpecificToEditor();

	/** Adds the words to the rule with the given token. */
	protected void addWordsToRule(List<String> words, WordRule rule, IToken token) {
		for(String word : words) {
			rule.addWord(word, token);
		}
	}
}
